{ copyright  Grahame Grieve
  refer to xyGraph.pas, xyGraph.hlp for more information }

unit Xyfile;

{$I xyDefine.pas}

interface

uses classes, graphics, xyData, xyGraph, Forms, wintypes;

procedure WriteAppToStream(w:TxyWriter; App:TAppearance);
procedure ReadAppFromStream(r:TxyReader; App:TAppearance);
procedure WriteDimToStream(w:TxyWriter; Dim:TDimensions);
procedure ReadDimFromStream(r:TxyReader; Dim:TDimensions);
procedure WriteLegToStream(w:TxyWriter; leg:TLegend);
procedure ReadLegFromStream(r:TxyReader; leg:TLegend);
procedure WriteAxsToStream(w:TxyWriter; axs:TAxis);
procedure ReadAxsFromStream(r:TxyReader; axs:TAxis);
procedure WriteSerToStream(w:TxyWriter; ser:Tseries; savedata:boolean);
procedure ReadSerFromStream(r:TxyReader; ser:Tseries);

const
  LogE = 0.4342944818;
 { time constants: Alert users will note that here and elsewhere where date/time
   values are used, small errors creep in due to assumptions implicit in the
   way TDateTime works. These have been tolerated in the interests of speed.
   Month and year are more variable: the only place where the length of these
   is approximated is for the guestimation of the number of ticks; I will
   be interested to hear if anybody has problems with this assumption }
  cminute1 = 1/(24 * 60);
  cminute5 = 5/(24 * 60);
  cminute30 = 30/(24 * 60);
  chour = (1/24);

procedure incmonth(var y,m:word; i:word);
function Pt(AX, AY: Integer): TPoint;

procedure AxisInitLogTicks(var FLogTickInfo:TLogTickInfo);

implementation

procedure incmonth(var y,m:word; i:word);
{ add i months to date xx/m/y }
begin
 if m + i > 12 then
   begin
   inc(y);
   m := m + i - 12;
   end
 else inc(m,i);
end;

function Pt(AX, AY: Integer): TPoint;
begin
  with Result do
  begin
    X := AX;
    Y := AY;
  end;
end;

procedure WriteAppToStream(w:TxyWriter; App:TAppearance);
begin
 with w,App do
  begin
  putstring (99, 'Section','Appearance');
  putboolean( 1, 'DuplicatesAllowed',AllowDuplicates);
  putinteger( 2, 'AxesColor',AxesColor);
  putinteger( 3, 'Color',BackgroundColor);
  putinteger( 4, 'MarginColor', MarginColor);
  putboolean( 5, 'PrintwithBackground', PrintBkgndColor);
{  putstring ( 6, 'Error Caption', ErrorCaption);}
{This is disabled: I think that this property should only be set
 by the programmer}
  putinteger( 7, 'GridColor', GridColor);
  putinteger( 8, 'GridLineStyle', longint(GridStyle)); {TPenStyle}
  putboolean( 9, 'GridLabelsShown', ShowGraphLabels);
  Putboolean(10, 'ShowPointsOutsidePlotArea', PlotOffGraph);
  putboolean(11, 'MarksAreShown', ShowMarks);
  putboolean(12, 'TicksAreShown', ShowTicks);
  putstring (13, 'TitleOfGraph', GraphTitle);
  putinteger(14, 'MinimumStepsShown', MinSteps);
  putinteger(15, 'MaximumStepsShown', MaxSteps);
  putfont   (16, 'FontForAxesTitles', TitleFont);
  putfont   (17, 'FontForTitle', CaptionFont);
  putfont   (18, 'FontForLabels', LabelFont);
  putboolean(19, 'ShowCrossAtZero', CrossAtZero);
  putinteger(20, 'ColorOfCross', CrossColor);
  putinteger(21, 'LengthOfCross', Crosslength);
  putboolean(22, 'PrintStyleforLines', PrintLineStyle);
  {you might want to disable PrintLineStyle ??}
  endsection;
  end;
end;

procedure ReadAppFromStream(r:TxyReader; App:TAppearance);
var finished:boolean;
    tempfont:TFont;
begin
 finished := false;
 tempfont := tfont.create;
 try
  with r, App do
   while not finished do
    case readblock of
      0: finished := true;
      1: AllowDuplicates := getboolean;
      2: AxesColor := getinteger;
      3: BackgroundColor  := getinteger;
      4: MarginColor := getinteger;
      5: PrintBkgndColor := getboolean;
      6: ErrorCaption := getstring;
      7: GridColor := getinteger;
      8: GridStyle :=  TPenStyle(getinteger);
      9: ShowGraphLabels := getboolean;
     10: PlotOffGraph := getboolean;
     11: ShowMarks := getboolean;
     12: ShowTicks := getboolean;
     13: GraphTitle := getstring;
     14: MinSteps := getinteger;
     15: MaxSteps := getinteger;
     16: begin
         getfont(tempfont);
         TitleFont := tempfont;
         end;
     17: begin
         getfont(tempfont);
         CaptionFont := tempfont;
         end;
     18: begin
         getfont(tempfont);
         LabelFont := tempfont;
         end;
     19: CrossAtZero := getboolean;
     20: CrossColor := getinteger;
     21: Crosslength := getinteger;
     22: PrintLineStyle := getboolean;
    else readunknown;
    end;
  finally
    tempfont.free;
  end;
end;

procedure WriteDimToStream(w:TxyWriter; Dim:TDimensions);
begin
 with w, Dim do
  begin
  putstring (99, 'Section:', 'Dimensions');
  putInteger( 1, 'BottomMargin',BottomMargin);
  putInteger( 2, 'LeftMargin',LeftMargin);
  PutInteger( 3, 'RightMargin',RightMargin);
  PutInteger( 4, 'TopMargin',TopMargin);
  PutInteger( 5, 'TickLength',TickLength);
  PutInteger( 6, 'XAxisTitleOffset',XAxisTitleOffset);
  PutInteger( 7, 'XAxisLabelOffset',XAxisLabelOffset);
  PutInteger( 8, 'YAxisTitleOffset',YAxisTitleOffset);
  PutInteger( 9, 'YAxisLabelOffset',YAxisLabelOffset);
  PutInteger(10, 'GraphTitleOffset',GraphTitleOffset);
  PutInteger(11, 'PrintXOffsetPct',PrintXOffsetPct);
  PutInteger(12, 'PrintYOffsetPct',PrintYOffsetPct);
  PutInteger(13, 'PrintScalePct',PrintScalePct);
  endsection;
  end;
end;

procedure ReadDimFromStream(r:TxyReader; Dim:TDimensions);
var finished:boolean;
begin
 finished := false;
 with r, Dim do
  while not finished do
   case readblock of
     0: finished := true;
     1: BottomMargin := getInteger;
     2: LeftMargin := getInteger;
     3: RightMargin := getInteger;
     4: TopMargin := getInteger;
     5: TickLength := getInteger;
     6: XAxisTitleOffset := getInteger;
     7: XAxisLabelOffset := getInteger;
     8: YAxisTitleOffset := getInteger;
     9: YAxisLabelOffset := getInteger;
    10: GraphTitleOffset := getInteger;
    11: PrintXOffsetPct := getInteger;
    12: PrintYOffsetPct := getInteger;
    13: PrintScalePct := getInteger;
  else readunknown;
  end;
end;

procedure WriteLegToStream(w:TxyWriter; leg:TLegend);
begin
 with w,leg do
  begin
  putstring (99, 'Section:','Legend');
  putboolean( 1, 'LegendIsVisible',visible);
  putinteger( 2, 'LegendColor',color);
  putinteger( 3, 'borderStyle',longint(borderStyle)); {TBorderStyle}
  putinteger( 4, 'LayoutMode',longint(Layout)); {TLegendStyle}
  putinteger( 5, 'Legendtop',top);
  putinteger( 6, 'Legendleft',left);
  putinteger( 7, 'Legendwidth',width);
  putinteger( 8, 'Legendheight',height);
  putfont   ( 9, 'Legendfont',font);
  putinteger(10, 'SpaceForSymbols',SymbolSpace);
  putinteger(11, 'XMargin',XMargin);
  putinteger(12, 'YMargin',YMargin);
  endsection;
  end;
end;

procedure ReadLegFromStream(r:TxyReader; leg:TLegend);
var finished:boolean;
    tempfont:TFont;
begin
 finished := false;
 tempfont := tfont.create;
 try
  with r, Leg do
   while not finished do
    case readblock of
      0: finished := true;
      1: visible := getboolean;
      2: color := getinteger;
      3: borderStyle := TBorderStyle(getinteger);
      4: Layout := TLegendStyle(getinteger);
      5: top := getinteger;
      6: left := getinteger;
      7: width := getinteger;
      8: height := getinteger;
      9: begin
         getfont(tempfont);
         Font := tempfont;
         end;
     10: SymbolSpace := getinteger;
     11: XMargin := getinteger;
     12: YMargin := getinteger;
    else readunknown;
    end;
  finally
    tempfont.free;
  end;
end;


procedure WriteAxsToStream(w:TxyWriter; axs:TAxis);
begin
 with w,axs do
  begin
  putstring (99, 'Section:', 'Axis');
  putstring ( 1, 'AxisTitle', Title);
  putinteger( 2, 'NumberOfDecimals', LabelDecimals);
  putboolean( 3, 'AdjustDecPlaces', AutoLabelDecimals);
  putinteger( 4, 'LogDivisions', LogCycleDivisions);
  putboolean( 5, 'useLogScale', LogScale);
  putfloat  ( 6, 'Max', Max);
  putfloat  ( 7, 'Min', Min);
  putfloat  ( 8, 'TickSpace', StepSize);
  putfloat  ( 9, 'MinimumScale', MinScaleLength);
  putboolean(10, 'ShowAsTime', ShowAsTime);
  putstring (11, 'TimeFormat', DateTimeFormat);
  putinteger(12, 'TimeTickMode', Longint(DateTickType));{dateticks}
  putboolean(13, 'AxisVisible', ShowAxis);
  putboolean(14, 'FlipAxis', Reversed);
  putinteger(15, 'OffsetMode', longint(OffsetType)); {TAxisOffsetType}
  putfloat  (16, 'OffSetValue', Offset);
  putboolean(17, 'ShowGridlines', Gridlines);

 {remember: the next 2 must come last! don't change the numbers, just
  put them in before these}
  putBoolean(18, 'AllowAutoSize', AutoSizing);
  putboolean(19, 'AllowTickRescale', AutoStepping);
  endsection;
  end;
end;

procedure ReadAxsFromStream(r:TxyReader; axs:TAxis);
var finished:boolean;
begin
 finished := false;
 with r, axs do
 while not finished do
  case readblock of
    0: finished := true;
    1: Title := getstring;
    2: LabelDecimals := getinteger;
    3: AutoLabelDecimals := getBoolean;
    4: LogCycleDivisions := getInteger;
    5: LogScale := getBoolean;
    6: Max := getfloat;
    7: Min := getfloat;
    8: StepSize := getfloat;
    9: MinScaleLength := getfloat;
   10: ShowAsTime := getboolean;
   11: DateTimeFormat := getstring;
   12: DateTickType := dateticks(getinteger);
   13: ShowAxis := getboolean;
   14: Reversed := getboolean;
   15: OffsetType := TAxisOffsetType(getinteger);
   16: Offset := getfloat;
   17: Gridlines := getBoolean;
   18: AutoSizing := getBoolean;
   19: AutoStepping := getboolean;
   else readunknown;
  end;
end;

procedure WriteSerToStream(w:TxyWriter; ser:Tseries; savedata:boolean);
begin
 with w,ser do
  begin
  putstring (99, '', 'Series');
  putBoolean( 1, '', Active);
  putBoolean( 2, '', DrawLine);
  putBoolean( 3, '', DrawPoints);
  putboolean( 4, '', FillPoints);
  putInteger( 5, '', LineColor);
  putinteger( 6, '', PointColor);
  putinteger( 7, '', BoundsColor);
  putinteger( 8, '', longint(PointShape)); {et_PointShape}
  putinteger( 9, '', longint(LineStyle)); {TPenStyle}
  putinteger(10, '', PointSize);
  if WhichYAxis = Graph.YAxis then
      putboolean(11, 'FirstAxis', true) else putboolean(11, 'FirstAxis', false);
  putfloat  (12, 'UpperBound', UpperBound);
  putfloat  (13, 'LowerBound', LowerBound);
  putinteger(14, 'BoundsMode', longint(BoundsType)); {et_BoundType}
  putinteger(15, 'BoundLineMode', longint(BoundsLineStyle)); {TPenStyle}
  putinteger(16, 'RegressionLineColor',RegressionLineColor);
  putinteger(17, 'RegressionLineStyle', longint(RegressionLineStyle)) {TPenStyle};
  putinteger(18, 'ShowInLegend', longint(LegendStatus)); {TLegendStatus}
  endsection;
  end;
 ser.savetostream(w,savedata);
end;

procedure ReadSerFromStream(r:TxyReader; ser:Tseries);
var finished:boolean;
begin
 finished := false;
 with r,ser do
  while not finished do
  case readblock of
   0 :begin
      ser.readfromstream(r);
      finished := true;
      end;
   1 :Active := getboolean;
   2 :DrawLine := getBoolean;
   3 :DrawPoints := getBoolean;
   4 :FillPoints := getBoolean;
   5 :LineColor := getinteger;
   6 :PointColor := getinteger;
   7 :BoundsColor := getinteger;
   8 :PointShape := et_PointShape(getinteger);
   9 :LineStyle := TPenStyle(getinteger);
   10:PointSize := getinteger;
   11:if getBoolean then WhichYAxis:= Graph.YAxis else WhichYAxis:= Graph.YAxis_second;
   12:UpperBound := getfloat;
   13:LowerBound := getFloat;
   14:BoundsType := et_BoundType(getinteger);
   15:BoundsLineStyle := TPenStyle(getinteger);
   16:RegressionLineColor := getinteger;
   17:RegressionLineStyle := TPenStyle(getinteger);
   18:LegendStatus := TLegendStatus(getinteger);
  else readunknown;
  end;
end;

procedure AxisInitLogTicks;
begin
  with FLogTickInfo do
  begin
    DrawMinorLabels := true;
    if LogTickCount > MaxLogTickCount then LogTickCount := MaxLogTickCount;
    LogTickArray[0] := 1;
   {CAREFUL! these LogTickCount values must correspond with AllowedLogTickCounts}
    case LogTickCount of
      0:
         begin
           LogTickCount := 0;
         end;
      1:
         begin
           LogTickCount := 1;
           LogTickArray[1] := 5;
         end;
      2, 3:
         begin
           LogTickCount := 2;
           LogTickArray[1] := 2;
           LogTickArray[2] := 5;
         end;
      4, 5, 6:
         begin
           LogTickCount := 5;
           LogTickArray[1] := 1.5;
           LogTickArray[2] := 2;
           LogTickArray[3] := 3;
           LogTickArray[4] := 5;
           LogTickArray[5] := 7;
         end;
      7, 8:
         begin
           LogTickCount := 7;
           LogTickArray[1] := 1.5;
           LogTickArray[2] := 2;
           LogTickArray[3] := 3;
           LogTickArray[4] := 4;
           LogTickArray[5] := 5;
           LogTickArray[6] := 6;
           LogTickArray[7] := 8;
         end;
      9: begin
           LogTickArray[1] := 1.2;
           LogTickArray[2] := 1.5;
           LogTickArray[3] := 2;
           LogTickArray[4] := 2.5;
           LogTickArray[5] := 3;
           LogTickArray[6] := 4;
           LogTickArray[7] := 5;
           LogTickArray[8] := 6;
           LogTickArray[9] := 8;
         end;
    end {case};
    LogTickArray[LogTickCount + 1] := 10;
      {This is a guard value for the linear searches in GetFirstTick and SetLogMinMax}
  end {with};
end {InitLogTicks};


end.
