{ copyright  Grahame Grieve
  refer to xyGraph.pas, xyGraph.hlp for more information }

{$I xyDefine.pas}
  
unit xyGetSrc;

interface

uses
  Wintypes, winprocs, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, DBTables, Db, Buttons, xyData, xyHolder;

type
  TxyGetDataSource = class(TForm)
    Notebook1: TNotebook;
    Label1: TLabel;
    ComboBox1: TComboBox;
    Label2: TLabel;
    ListBox1: TListBox;
    Label3: TLabel;
    ComboBox2: TComboBox;
    Label4: TLabel;
    Memo1: TMemo;
    ListBox2: TListBox;
    Label5: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    Label6: TLabel;
    ListBox3: TListBox;
    Label7: TLabel;
    ComboBox3: TComboBox;
    Memo2: TMemo;
    Label8: TLabel;
    procedure FormShow(Sender: TObject);
    procedure ComboBox1Change(Sender: TObject);
    procedure AllDblClick(Sender: TObject);
  private
    { Private declarations }
    procedure updateTablesPage;
    procedure updateQueryPage;
    procedure updateDataSourcesPage;
    procedure updatexyHoldersPage;
    procedure updateODBCPage;
  public
    { Public declarations }
    CurrSource:string;
  end;

var
  xyGetDataSource: TxyGetDataSource;

implementation

{$IFDEF ODBCEXPRESS} uses OCIH, OCI, ODI, OCL, OVCL; {$ENDIF}
{$R *.DFM}


procedure TxyGetDataSource.FormShow(Sender: TObject);
begin
  case notebook1.PageIndex of
    0:updateTablesPage;
    1:updateQueryPage;
    2:updateDataSourcesPage;
    3:updatexyHoldersPage;
    4:updateODBCPage;
  end;
end;

procedure TxyGetDataSource.updateTablesPage;
var alias,details:string;
begin
  Session.GetAliasNames(combobox1.items);
  if combobox1.items.count > 0 then
      combobox1.itemindex := 0;
  SourceSplit(CurrSource, alias, details);
  if alias <> '' then combobox1.ItemIndex := combobox1.items.indexof(alias);
  Session.GetTableNames(combobox1.items[combobox1.itemindex], '',
                        false, false, listbox1.items);
  if details <> '' then
     listbox1.itemindex := listbox1.Items.IndexOf(details);
end;

procedure TxyGetDataSource.updateQueryPage;
var alias,details:string;
begin
  Session.GetAliasNames(combobox2.items);
  if combobox2.items.count > 0 then
      combobox2.itemindex := 0;
  SourceSplit(CurrSource, alias, details);
  memo1.text := details;
end;

procedure TxyGetDataSource.ComboBox1Change(Sender: TObject);
begin
  Session.GetTableNames(combobox1.items[combobox1.itemindex], '',
                        false, false, listbox1.items);
end;

procedure TxyGetDataSource.updateDataSourcesPage;
var iform, iComp:integer;
begin
  { look through all forms for any datasource with a name matching datasourcename
    and then get it's dataset}
  {$IFDEF WIN32}
  //both 32bit delphi's have datamodules as well. check these first
  for iForm := 0 to screen.DataModulecount - 1 do
    with screen.datamodules[iForm] do
      for iComp := 0 to ComponentCount - 1 do
        if (Components[iComp] is TDataSource) then listbox2.items.add(components[iComp].name);
  {$ENDIF}
  for iForm := 0 to screen.Formcount - 1 do
    with screen.Forms[iForm] do
      for iComp := 0 to ComponentCount - 1 do
        if (Components[iComp] is TDataSource) then listbox2.items.add(components[iComp].name);
  if CurrSource <> '' then
    listbox2.itemindex := listbox2.items.indexof(CurrSource);
end;

procedure TxyGetDataSource.updatexyHoldersPage;
var iform, iComp:integer;
begin
  {$IFDEF WIN32}
  //both 32bit delphi's have datamodules as well. check these first
  for iForm := 0 to screen.DataModulecount - 1 do
    with screen.datamodules[iForm] do
      for iComp := 0 to ComponentCount - 1 do
        if (Components[iComp] is TxyHolder) then listbox3.items.add(TxyHolder(components[iComp]).ID);
  {$ENDIF}
  for iForm := 0 to screen.Formcount - 1 do
    with screen.Forms[iForm] do
      for iComp := 0 to ComponentCount - 1 do
        if (Components[iComp] is TxyHolder) then listbox3.items.add(TxyHolder(components[iComp]).ID);
  if CurrSource <> '' then
    listbox3.itemindex := listbox3.items.indexof(CurrSource);
end;

procedure TxyGetDataSource.updateODBCPage;
{$IFDEF ODBCEXPRESS}
var HBase:THbase;
    HEnv:THEnv;
    HDS:TOEDataSource;
    iname, idesc, alias, details:string;
    i,j:integer;
begin
  SourceSplit(CurrSource, alias, details);
  HEnv := THEnv.create(nil);
  HDS := TOEDataSource.create(nil);
  HDS.HEnv := HEnv;
  j := 0;
  i := 0;
  with HDS do
    if GetFirst(iname, idesc) then
      begin
      if iname = alias then j := i;
      combobox3.items.add(iname + '('+idesc+')');
      inc(i);
      while getNext(iname, idesc) do
        begin
        if iname = alias then j := i;
        combobox3.items.add(iname + '('+idesc+')');
        end;
      end;
  HDS.free;
  HEnv.free;
  combobox3.itemindex := j;
  memo2.text := details;
end;
{$ELSE}
begin
end;
{$ENDIF}

procedure TxyGetDataSource.AllDblClick(Sender: TObject);
begin
  modalresult := mrOK;
end;



end.
